/***************************************************************************
 *   Copyright (C) 2004 by Diego 'Flameeyes' Petteno                       *
 *   dgp85@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "sfvfile.h"

#include <zlib.h>

#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qapplication.h>
#include <qregexp.h>

namespace KDigest {

/**
@brief Default constructor
@param path Path of the SFV file
@param create If true, the SFV file will be created
@param parent parent object (passed to DigestFile and then to QObject)
@param name object's name (passed to DigestFile and then to QObject)
*/
SFVfile::SFVfile(const QString &path, bool create, QObject *parent, const char *name)
	: DigestFile(parent, name)
{
	if ( ! create )
	{
		QFile f(path);
		if ( ! f.open(IO_ReadOnly) ) {
			///@todo Thrown an error
			return;
		}
		
		QTextStream ts(&f);
		
		QString line;
		QRegExp digest("^(.*) ([0-9a-fA-F]{1,8})[ ]*$");
		QRegExp comment("^[ ]*;(.*)$");
		while(!ts.atEnd())
		{
			line = ts.readLine();
			if ( line == QString::null )
				continue;
			
			if ( comment.search(line) != -1 )
				m_comments += comment.cap(1);
			
			if ( digest.search(line) == -1 )
				continue;
			
			SFVentry *e = new SFVentry;
			
			e->filename = digest.cap(1);
			e->status = dsNotChecked;
			
			e->crc32 = digest.cap(2).toUInt(NULL, 16);
			e->aCrc32 = 0;
			
			m_entries.append(e);
		}
		
		f.close();
	} else {
		///@todo write
	}
}

/**
@brief Check the entries loaded from the SFV file for exactness and update the m_entries list
@param path Path to apply to the entries' filenames to get the complete path
*/
bool SFVfile::testFile(const QString &path)
{
	int i = -1;
	bool allok = true;
	QDir currDir(path);
	for( DigestEntries::iterator it = m_entries.begin(); it != m_entries.end(); it++ )
	{
		SFVentry *curr = reinterpret_cast<SFVentry *>(*it);
		if ( ! currDir.exists( curr->filename ) )
		{
			allok = false;
			curr->status = dsNotFound;
			continue;
		}
		
		QFile chkFile( path + '/' + curr->filename );
		if ( ! chkFile.open(IO_ReadOnly) )
		{
			allok = false;
			curr->status = dsErrorAccess;
			continue;
		}
		
		int read = 0;
		uint32_t crc = crc32(0L, Z_NULL, 0);
		
		while( (read = chkFile.readBlock((char*)m_buffer, 4096) ) )
		{
			qApp->processEvents();
			crc = crc32(crc, m_buffer, read);
		}
		
		curr->aCrc32 = crc;
		if ( crc == curr->crc32 )
			curr->status = dsOk;
		else
		{
			curr->status = dsBadDigest;
			allok = false;
		}
		emit cycleCompleted(++i);
	}
	emit cycleCompleted(++i);
	
	return allok;
}

SFVfile::~SFVfile()
{
}

};
